# LineConvertNX
#

require "nkf"

module TDS2TDModule
	#
	# iɓ
	def enterParagraph
		unless @is_paragraph
			@is_paragraph = true
			"<p>"
		else
			""
		end
	end

	#
	# i甲
	def leaveParagraph
		if @is_paragraph
			@is_paragraph = false
			"</p>\n"
		else
			""
		end
	end

	#
	# CC^O\bh
	def tag_inline(tag_type, tagtext_open, tagtext_close, text)
		case tag_type
		when :tag_open
			enterParagraph << tagtext_open
		when :tag_normal
			enterParagraph << "#{tagtext_open}#{text}#{tagtext_close}"
		when :tag_close
			tagtext_close
		end
	end

	#
	# ubN^O\bh
	def tag_block(tag_type, tagtext_open, tagtext_close, text = nil)
		case tag_type
		when :tag_open, :tag_normal
			tag_text = leaveParagraph
			@is_paragraph = true
			tag_text << "#{tagtext_open}\n"
		when :tag_close
			@is_paragraph = false
			"#{tagtext_close}\n"
		end
	end

	#
	# Qƕϊ\bh
	CharRefMap = [
		['&', '&amp;'],
		['<', '&lt;'],
		['>', '&gt;'],
		['"', '&quot;']
	
	]
	def convertCharRef!(text)
		CharRefMap.each {|ch|
			text.gsub!(ch[0], ch[1])
		}
	end

	
	#
	# AJ[ϊ\bh
	def convertADic!(text)
		@dic.each{ |word|
			text.gsub!(word, '<%=a "\0" %>')
		}
	end
	
end


#
# sϊ
class LineConvert
	include TDS2TDModule


	#
	# sϊ\bh
	attr_reader :convert

	#
	# sϊ\bh
	attr_reader :is_paragraph

	#
	# 
	def initialize
		@is_paragraph = false					# iH(<p>^O)
		@dic = []								# AJ[

		@conv_stack = []						# ϊ[`X^bN
		@convert = Proc.new {|line|				# sϊ[`
			case line
			when /^([A-Z]+)\* (.+)/o
				# CMD* link text `s
				tmp = @is_paragraph				# ꎞIɒiɐݒ
				@is_paragraph = true
				link_text = cmd_LINK(:tag_normal, $2)
				@is_paragraph = tmp
				eval( "cmd_#{$1}(:tag_normal, link_text)" )

			when /^\/([A-Z]+)/o
				# /CMD `s
				eval( "cmd_#{$1}(:tag_close)" )

			when /^([A-Z]+)\+/o
				# CMD+ `s
				eval( "cmd_#{$1}(:tag_open)" )

			when /^([A-Z]+)( (.*)|\s*$)/o
				# CMD text `s
				eval( "cmd_#{$1}(:tag_normal, $3)" )

			when /^\s*$/o
				# 󔒍s
				@conv_stack.push @convert

				# 󔒍sǂݔ΂[`ɕύX
				@convert = Proc.new {|line|
					case line
					when /\S/o
						@convert = @conv_stack.pop
						@convert.call(line)
					else
						""
					end
				}
				leaveParagraph
			else
				# ʏs
				enterParagraph << line.chomp
			end
		}
	end

	#
	# ^O
	#
	def cmd_NEW(tag_type, text = nil)
		tag_text = leaveParagraph
		tag_text << "\n"
		tag_text << "<" if /^</o =~ text
		tag_text << "#{text}\n"
		tag_text << enterParagraph
	end

	def cmd_SUB(tag_type, text = nil)
		cmd_NEW(tag_type, text)			# NEW^OƓƂ݂Ȃ
	end


	#
	# CC^O
	#
	def cmd_LINK(tag_type, text = nil)
		uri , link_text = text.match(/^(\S+) (.+)/)[1..2]
		tag_inline(tag_type, %Q(<a href="#{uri}">), '</a>', link_text)
	end

	def cmd_STRONG(tag_type, text = nil)
		tag_inline(tag_type, '<strong>', '</strong>', text)
	end

	def cmd_EM(tag_type, text = nil)
		tag_inline(tag_type, '<em>', '</em>', text)
	end

	def cmd_LI(tag_type, text = nil)
		tag_inline(tag_type, '<li>', "</li>\n", text)
	end

	def cmd_STRIKE(tag_type, text = nil)
		tag_inline(tag_type, '<del>', '</del>', text)
	end

	def cmd_BIG(tag_type, text = nil)
		tag_inline(tag_type, '<span style="font-size: x-large">', '</span>', text)
	end

	def cmd_SMALL(tag_type, text = nil)
		tag_inline(tag_type, '<span style="font-size: x-small">', '</span>', text)
	end



	#
	# ubN^O
	#
	def cmd_OL(tag_type, text = nil)
		tag_block(tag_type, "<ol>", "</ol>", text)
	end

	def cmd_UL(tag_type, text = nil)
		tag_block(tag_type, "<ul>", "</ul>", text)
	end

	def cmd_CODE(tag_type, text = nil)
		@conv_stack.push @convert
		@convert = Proc.new {|line|
			case line
			when /^\/CODE/o
				# /CODE s
				@convert = @conv_stack.pop
				"</code></pre>\n"
			when /^\s*$/o
				# s
				cmd_BR
			else
				# ŝ܂܏o
				line
			end
		}
		leaveParagraph << "<pre><code>"
	end

	def cmd_PRE(tag_type, text = nil)
		@conv_stack.push @convert
		@convert = Proc.new {|line|
			case line
			when /^\/PRE/o
				# /CODE s
				@convert = @conv_stack.pop
				"</pre>\n"
			when /^\s+$/o
				# s
				cmd_BR
			else
				# ŝ܂܏o
				line
			end
		}
		leaveParagraph << "<pre>"
	end


	#
	# vf^O
	#
	def cmd_BR(tag_type = nil, text = "")
		text = "" unless text					# nilnꂽꍇ
		clear = text.match(/clear=(\w+)/).to_a[1]
		if clear
			%Q(<br style="clear: #{clear}">\n)
		else
			"<br>\n"
		end
	end

	def cmd_IMG(tag_type = nil, text = "")
		src, align, width, height, alt = text.split(' ', 5)
		case align
		when "right", "left"
			%Q(<img src="#{src}" style="float: #{align}" width="#{width}" height="#{height}" alt="#{alt}">)
		else
			%Q(<img src="#{src}" align="#{align}" width="#{width}" height="#{height}" alt="#{alt}">)
		end
	end

end


#
# sϊ(Ǝ^O)
class LineConvertEx < LineConvert
	#
	# 
	def initialize
		super
		
		# AJ[(orK\ŋLq)
		@dic = [
			# c[
			'bc4mpeg4',
			'upftp.pl',
			'upftp2.pl',
			'upftp.rb',
			'asi.pl',
			'asi.rb',
			'aaa.rb',
			'any.rb',
		]
	end
	
	
	#
	# Ǝ^Odl͈ȉɒǉƂ悢

	def cmd_NEW(tag_type, text = nil)
		# ȉ̕ϊs
		#
		# (TDS)
		#	NEW BGM
		#	PRE
		#	uȖv by ̎薼
		#	/PRE
		#
		# (tDiary)
		#	[] Ȗ / ̎薼
		#
		if text == "BGM"
			@conv_stack.push @convert
			@convert = Proc.new {|line|
				case line
				when /^\/PRE/o
					# /CODE s
					@convert = @conv_stack.pop
					enterParagraph
				when /u(.*)v\s*by\s*(.*)/o
					"[] #{$1} / #{$2}\n"
				else
					# o͂Ȃ
					""
				end
			}
			leaveParagraph << "\n"
		else
			super
		end
	end

end

